@@ -3,10 +3,14 @@ from contextlib import contextmanager
 import oracledb
 from oracledb import SessionPool, DatabaseError, Cursor
 
-oracledb.init_oracle_client()
-# lib_dir="C:\\oracle"
 import ipih
 from pih import A, PIHThread
+from PolibaseService.const import HOST
+
+#if not A.D.contains(A.SYS.host(), HOST.name):
+#oracledb.init_oracle_client(lib_dir=r"C:\instantclient")
+
+
 from pih.consts.errors import Error
 from PolibaseService.const import *
 from pih.collections import (
@@ -21,12 +25,12 @@ from pih.tools import (
     nn,
     ne,
     nl,
+    nnt,
     one,
     esc,
     escs,
     BitMask as BM,
     while_not_do,
-    
 )
 
 from typing import Any, Callable
@@ -50,9 +54,9 @@ def get_dsn(test: bool) -> str:
 class PolibaseApi:
     pool_map: dict[bool, SessionPool] = {}
     # handlers
-    on_note_emailed_update_handler: Callable[[int, int], None]
-    on_person_creation_or_update_handler: Callable[[dict[str, Any], bool], None]
-    on_person_saldo_update_handler: Callable[[dict[str, float | int]], None]
+    on_note_emailed_update_handler: Callable[[int, int], None] | None = None
+    on_person_creation_or_update_handler: Callable[[dict[str, Any], bool], None] | None = None
+    on_person_saldo_update_handler: Callable[[dict[str, float | int]], None] | None = None
 
     @staticmethod
     def init(debug: bool) -> None:
@@ -623,7 +627,7 @@ class PolibaseApi:
                     j((PERSON_TELEPHONE_NUMBER, index, " = ", escs(value)))
                 )
             )
-            result = PolibaseApi.fill_data(fetch_result, cursor.description)
+            result = nnt(PolibaseApi.fill_data(fetch_result, cursor.description))
         return result
 
     @staticmethod
@@ -637,7 +641,7 @@ class PolibaseApi:
                     j((PERSON_EMAIL, "=", escs(value)))
                 )
             )
-            result = PolibaseApi.fill_data(fetch_result, cursor.description)
+            result = nnt(PolibaseApi.fill_data(fetch_result, cursor.description))
         return result
 
     @staticmethod
@@ -655,7 +659,7 @@ class PolibaseApi:
                     )
                 )
             )
-            result = PolibaseApi.fill_data(fetch_result, cursor.description)
+            result = nnt(PolibaseApi.fill_data(fetch_result, cursor.description))
         return result
 
     @staticmethod
@@ -748,7 +752,7 @@ class PolibaseApi:
                 f"select {PolibaseApi.get_person_pin_field()}, {PolibaseApi.get_person_barcode_field()} from {PERSON_TABLE_NAME} where {PERSON_NO}=: pin",
                 pin=pin,
             )
-            result = PolibaseApi.fill_data(fetch_result, cursor.description)
+            result = nnt(PolibaseApi.fill_data(fetch_result, cursor.description))
         return result
 
     @staticmethod
@@ -779,7 +783,7 @@ class PolibaseApi:
                     )
                 )
             )
-            result = PolibaseApi.fill_data(fetch_result, cursor.description)
+            result = nnt(PolibaseApi.fill_data(fetch_result, cursor.description))
         return result
 
     @staticmethod
@@ -920,6 +924,31 @@ class PolibaseApi:
             )
         )
 
+    @staticmethod
+    def get_person_saldo_after_id(
+        value: int, test: bool | None = None
+    ) -> list[dict[str, int | float]]:
+        result_data: list[dict[str, int | float]] = []
+        with PolibaseApi.get_cursor(test) as cursor:
+            result_data.extend(
+                PolibaseApi.fill_data(
+                    cursor.execute(
+                        PolibaseApi.get_person_saldo_query_with_condition(
+                            for_cqn=False,
+                            value=js(
+                                (
+                                    j((PATIENT_SALDO_PREFIX, NUMBER)),
+                                    ">",
+                                    value,
+                                )
+                            ),
+                        )
+                    ),
+                    cursor.description,  # type: ignore
+                )
+            )  # type: ignore
+            return result_data
+
     @staticmethod
     def get_note_emailed_query(fields: list[str] | None = None) -> str:
         return PolibaseApi.get_note_emailed_query_with_condition(None, fields)
@@ -956,42 +985,47 @@ class PolibaseApi:
     def get_person_pin_list_by_visit_date(
         date: str, only_operation_department: bool = False, test: bool | None = None
     ) -> list[int]:
-        complete_list_part: str = """OJBS_COMPLETE_LIST as (
-                                                            select
-                                                            DIV_NAME,
-                                                            DIV_NOTES DEPARTMENT,
-                                                            JOB_NO,
-                                                            JOB_NAME,
-                                                            ORD_PAT_NO,
-                                                            PATIENT.PER_FULL_NAME,
-                                                            PATIENT.PER_NO,
-                                                            OJB_DOC_NO as DOC_NO,
-                                                            DOCTOR.PER_FULL_NAME as DOCTOR,
-                                                            OJB_MED_NO as MED_NO,
-                                                            DOCTOR.PER_PLI_NO as DOC_PLI_NO,
-                                                            OJB_NO,
-                                                            OJB_ORD_NO ORDER_NO,
-                                                            ORD_STRT CREATE_AT,
-                                                            ORD_PAID PAID_AT,
-                                                            ORD_TOTAL_PRICE TOTAL_PRICE,
-                                                            OJB_JOB_PRC,
-                                                            OJB_JOB_QNT,
-                                                            OJB_JOB_KO,
-                                                            OJB_DOC_PRC DOC_RATE,
-                                                            OJB_MED_PRC MED_RATE,
-                                                            JSL_COST_DOC,
-                                                            JSL_NO,
-                                                            JSL_TRF_NO
-                                                            from DIVS_ALL
-                                                            left outer join JOBLISTS on JLI_DIV_NO=DIV_NO
-                                                            left outer join JOBLIS on JIL_JLI_NO=JLI_NO
-                                                            left outer join JOBS on JIL_JOB_NO=JOB_NO
-                                                            left outer join OJBS on OJB_JOB_NO=JOB_NO
-                                                            left outer join PERSONS DOCTOR on OJB_DOC_NO=DOCTOR.PER_NO
-                                                            left outer join PERSONS PATIENT on OJB_PAT_NO=PATIENT.PER_NO
-                                                            left outer join JOBSALS ON JSL_JOB_NO=OJB_JOB_NO and DOCTOR.PER_PLI_NO=JSL_PLI_NO
-                                                        )"""
+        complete_list_part: str = """
+                            OJBS_COMPLETE_LIST as (
+                            select
+                            DIV_NAME,
+                            DIV_NOTES DEPARTMENT,
+                            JOB_NO,
+                            JOB_NAME,
+                            ORD_PAT_NO,
+                            PATIENT.PER_FULL_NAME,
+                            PATIENT.PER_NO,
+                            OJB_DOC_NO as DOC_NO,
+                            DOCTOR.PER_FULL_NAME as DOCTOR,
+                            OJB_MED_NO as MED_NO,
+                            DOCTOR.PER_PLI_NO as DOC_PLI_NO,
+                            OJB_NO,
+                            OJB_ORD_NO ORDER_NO,
+                            ORD_STRT CREATE_AT,
+                            ORD_PAID PAID_AT,
+                            ORD_TOTAL_PRICE TOTAL_PRICE,
+                            OJB_JOB_PRC,
+                            OJB_JOB_QNT,
+                            OJB_JOB_KO,
+                            OJB_DOC_PRC DOC_RATE,
+                            OJB_MED_PRC MED_RATE,
+                            JSL_COST_DOC,
+                            JSL_NO,
+                            JSL_TRF_NO
+                            from DIVS_ALL
+                            left outer join JOBLISTS on JLI_DIV_NO=DIV_NO
+                            left outer join JOBLIS on JIL_JLI_NO=JLI_NO
+                            left outer join JOBS on JIL_JOB_NO=JOB_NO
+                            left outer join OJBS on OJB_JOB_NO=JOB_NO
+                            left outer join PERSONS DOCTOR on OJB_DOC_NO=DOCTOR.PER_NO
+                            left outer join PERSONS PATIENT on OJB_PAT_NO=PATIENT.PER_NO
+                            left outer join JOBSALS ON JSL_JOB_NO=OJB_JOB_NO and DOCTOR.PER_PLI_NO=JSL_PLI_NO
+                        )
+                        """
 
+        pin_list: list[int] = A.S.get(
+            A.CT_S.POLIBASE_PERSON_REVIEW_NOTIFICATION_DOCTOR_PERSON_PIN_LIST
+        )
         query: str = js(
             (
                 (
@@ -1077,15 +1111,11 @@ class PolibaseApi:
                 ),
                 (
                     None
-                    if only_operation_department
+                    if only_operation_department or e(pin_list)
                     else j(
                         (
                             "and DOC_NO IN(",
-                            A.D.list_to_string(
-                                A.S.get(
-                                    A.CT_S.POLIBASE_PERSON_REVIEW_NOTIFICATION_DOCTOR_PERSON_PIN_LIST
-                                )
-                            ),
+                            A.D.list_to_string(A.D.map(str, pin_list)),
                             ")",
                         )
                     )
